@@ -1,19 +1,19 @@
-/** 
- * goaccess.c -- main log analyzer 
- * Copyright (C) 2010 by Gerardo Orellana <goaccess@prosoftcorp.com>
- * GoAccess - An ncurses apache weblog analyzer & interactive viewer
+/**
+ * goaccess.c -- main log analyzer
+ * Copyright (C) 2009-2012 by Gerardo Orellana <goaccess@prosoftcorp.com>
+ * GoAccess - An Ncurses apache weblog analyzer & interactive viewer
  *
- * This program is free software; you can redistribute it and/or    
- * modify it under the terms of the GNU General Public License as   
- * published by the Free Software Foundation; either version 2 of   
- * the License, or (at your option) any later version.              
- *                                                                  
- * This program is distributed in the hope that it will be useful,  
- * but WITHOUT ANY WARRANTY; without even the implied warranty of   
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the    
- * GNU General Public License for more details.                     
- *                                                                  
- * A copy of the GNU General Public License is attached to this 
+ * This program is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU General Public License as
+ * published by the Free Software Foundation; either version 2 of
+ * the License, or (at your option) any later version.
+ *
+ * This program is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
+ *
+ * A copy of the GNU General Public License is attached to this
  * source distribution for its full text.
  *
  * Visit http://goaccess.prosoftcorp.com for new releases.
@@ -51,6 +51,7 @@
 #include "alloc.h"
 #include "commons.h"
 #include "error.h"
+#include "output.h"
 #include "parser.h"
 #include "settings.h"
 #include "ui.h"
@@ -83,6 +84,7 @@ init_struct (void)
    logger->alloc_counter = 0;
    logger->counter = 0;
    logger->current_module = 1;
+   logger->resp_size = 0LL;
    logger->total_invalid = 0;
    logger->total_process = 0;
 
@@ -94,20 +96,17 @@ cmd_help (void)
 {
    printf ("\nGoAccess - %s\n\n", GO_VERSION);
    printf ("Usage: ");
-   printf ("goaccess [ -b ][ -s ][ -e IP_ADDRESS][ - a ]< -f log_file >\n\n");
+   printf ("goaccess [ -e IP_ADDRESS][ - a ][ - c ]< -f log_file >\n\n");
    printf ("The following options can also be supplied to the command:\n\n");
    printf (" -f <argument> - Path to input log file.\n");
-   printf (" -b            - Enable total bandwidth consumption.\n");
-   printf ("                 For faster parsing, don't enable this flag.\n");
-   printf (" -s            - Enable HTTP status codes report.\n");
-   printf ("                 For faster parsing, don't enable this flag.\n");
+   printf (" -c            - Prompt log/date configuration window.\n");
    printf (" -a            - Enable a List of User-Agents by host.\n");
    printf ("                 For faster parsing, don't enable this flag.\n");
    printf (" -e <argument> - Exclude an IP from being counted under the\n");
    printf ("                 HOST module. Disabled by default.\n\n");
    printf ("Examples can be found by running `man goaccess`.\n\n");
    printf ("For more details visit: http://goaccess.prosoftcorp.com\n");
-   printf ("GoAccess Copyright (C) 2009-2010 GNU GPL'd, by Gerardo Orellana");
+   printf ("GoAccess Copyright (C) 2009-2012 GNU GPL'd, by Gerardo Orellana");
    printf ("\n\n");
    exit (EXIT_FAILURE);
 }
@@ -165,7 +164,7 @@ allocate_structs (int free_me)
    /* allocate 10 per module */
    MALLOC_STRUCT (s_display, 170);
 
-   /* note that the order in which we call them, 
+   /* note that the order in which we call them,
     * is the way modules will be displayed */
    generate_unique_visitors (s_display, logger);
 
@@ -191,14 +190,9 @@ allocate_structs (int free_me)
    MALLOC_STRUCT (s_holder, g_hash_table_size (ht_hosts));
    generate_struct_data (ht_hosts, s_holder, s_display, logger, 8);
 
-   if (!http_status_code_flag)
-      goto nohttpstatuscode;
-
    MALLOC_STRUCT (s_holder, g_hash_table_size (ht_status_code));
    generate_struct_data (ht_status_code, s_holder, s_display, logger, 9);
 
- nohttpstatuscode:;
-
    MALLOC_STRUCT (s_holder, g_hash_table_size (ht_referring_sites));
    generate_struct_data (ht_referring_sites, s_holder, s_display, logger, 10);
 
@@ -240,8 +234,8 @@ render_screens (void)
    wattroff (header_win, COLOR_PAIR (BLUE_GREEN));
    wrefresh (header_win);
    /* no valid entries to process from the log */
-   if ((logger->total_process == 0)
-       || (logger->total_process == logger->total_invalid))
+   if ((logger->total_process == 0) ||
+       (logger->total_process == logger->total_invalid))
       error_handler (__PRETTY_FUNCTION__, __FILE__, __LINE__,
                      "Nothing valid to process.");
 }
@@ -339,8 +333,7 @@ get_keys (void)
           break;
           /* key 9 - module 9 */
        case 57:
-          if (http_status_code_flag)
-             logger->current_module = 9;
+          logger->current_module = 9;
           update_header (header_win, logger->current_module);
           break;
           /* key 0 - module 10 */
@@ -390,6 +383,16 @@ get_keys (void)
           refresh ();
           render_screens ();
           break;
+       case 98:                /* SCRL DOWN */
+          werase (main_win);
+          scrolling.init_scrl_main_win = logger->alloc_counter - real_size_y;
+          render_screens ();
+          break;
+       case 116:               /* SCRL UP */
+          werase (main_win);
+          scrolling.init_scrl_main_win = 0;
+          render_screens ();
+          break;
        default:
           if (piping)
              break;
@@ -401,7 +404,7 @@ get_keys (void)
                                __LINE__, "Unable to read log file.");
              if (!fseeko (fp, size1, SEEK_SET))
                 while (fgets (buf, BUFFER, fp) != NULL)
-                   parse_log (logger, ifile, buf);
+                   parse_log (logger, ifile, buf, -1);
              fclose (fp);
              size1 = size2;
              allocate_structs (1);
@@ -422,10 +425,10 @@ main (int argc, char *argv[])
 {
    extern char *optarg;
    extern int optind, optopt, opterr;
-   int row, col, o, index;
+   int row, col, o, index, conf = 0, quit = 0;
 
    opterr = 0;
-   while ((o = getopt (argc, argv, "f:e:sba")) != -1) {
+   while ((o = getopt (argc, argv, "f:e:ac")) != -1) {
       switch (o) {
        case 'f':
           ifile = optarg;
@@ -433,15 +436,12 @@ main (int argc, char *argv[])
        case 'e':
           ignore_host = optarg;
           break;
-       case 's':
-          http_status_code_flag = 1;
-          break;
-       case 'b':
-          bandwidth_flag = 1;
-          break;
        case 'a':
           host_agents_list_flag = 1;
           break;
+       case 'c':
+          conf = 1;
+          break;
        case '?':
           if (optopt == 'f' || optopt == 'e')
              fprintf (stderr, "Option -%c requires an argument.\n", optopt);
@@ -454,6 +454,8 @@ main (int argc, char *argv[])
           abort ();
       }
    }
+   if (!isatty (STDOUT_FILENO))
+      outputting = 1;
    if (ifile != NULL && !isatty (STDIN_FILENO))
       cmd_help ();
    if (ifile == NULL && isatty (STDIN_FILENO))
@@ -514,10 +516,15 @@ main (int argc, char *argv[])
       g_hash_table_new_full (g_str_hash, g_str_equal,
                              (GDestroyNotify) g_free, g_free);
 
-    /** 'should' work on UTF-8 terminals as long as the 
+    /** 'should' work on UTF-8 terminals as long as the
      ** user did set it to *._UTF-8 locale **/
     /** ###TODO: real UTF-8 support needs to be done **/
    setlocale (LC_CTYPE, "");
+   parse_conf_file ();
+
+   if (outputting)
+      goto out;
+
    initscr ();
    clear ();
    if (has_colors () == FALSE)
@@ -525,7 +532,6 @@ main (int argc, char *argv[])
                      "Your terminal does not support color");
    set_input_opts ();
    start_color ();
-   parse_conf_file ();
    init_colors ();
 
    attron (COLOR_PAIR (COL_WHITE));
@@ -546,10 +552,16 @@ main (int argc, char *argv[])
       error_handler (__PRETTY_FUNCTION__, __FILE__, __LINE__,
                      "Unable to allocate memory for new window.");
 
+ out:
+   logger = init_struct ();
+   if (isatty (STDIN_FILENO) && (log_format == NULL || conf) && !outputting) {
+      refresh ();
+      quit = verify_format (logger);
+   }
+
    /* main processing event */
    (void) time (&start_proc);
-   logger = init_struct ();
-   if (parse_log (logger, ifile, NULL))
+   if (!quit && parse_log (logger, ifile, NULL, -1))
       error_handler (__PRETTY_FUNCTION__, __FILE__, __LINE__,
                      "Error while processing file");
    allocate_structs (0);
@@ -557,21 +569,32 @@ main (int argc, char *argv[])
    (void) time (&end_proc);
 
    initial_reqs = logger->total_process;
+
+   if (outputting) {
+      /* no valid entries to process from the log */
+      if ((logger->total_process == 0) ||
+          (logger->total_process == logger->total_invalid))
+         goto done;
+
+      (void) output_html (logger);
+      goto done;
+   }
    /* draw screens */
-   int x, y;
-   getmaxyx (main_win, y, x);
+   int y = getmaxy (main_win);
 
    scrolling.scrl_main_win = y;
    scrolling.init_scrl_main_win = 0;
    render_screens ();
    get_keys ();
 
-   write_conf_file ();
-   house_keeping (logger, s_display);
    attroff (COLOR_PAIR (COL_WHITE));
 
-   /* restore tty modes and reset 
+   /* restore tty modes and reset
     * terminal into non-visual mode */
    endwin ();
+ done:
+   write_conf_file ();
+   house_keeping (logger, s_display);
+
    return 0;
 }
