@@ -1,19 +1,19 @@
-/** 
+/**
  * ui.c -- curses user interface
- * Copyright (C) 2010 by Gerardo Orellana <goaccess@prosoftcorp.com>
- * GoAccess - An ncurses apache weblog analyzer & interactive viewer
+ * Copyright (C) 2009-2012 by Gerardo Orellana <goaccess@prosoftcorp.com>
+ * GoAccess - An Ncurses apache weblog analyzer & interactive viewer
  *
- * This program is free software; you can redistribute it and/or    
- * modify it under the terms of the GNU General Public License as   
- * published by the Free Software Foundation; either version 2 of   
- * the License, or (at your option) any later version.              
- *                                                                  
- * This program is distributed in the hope that it will be useful,  
- * but WITHOUT ANY WARRANTY; without even the implied warranty of   
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the    
- * GNU General Public License for more details.                     
- *                                                                  
- * A copy of the GNU General Public License is attached to this 
+ * This program is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU General Public License as
+ * published by the Free Software Foundation; either version 2 of
+ * the License, or (at your option) any later version.
+ *
+ * This program is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
+ *
+ * A copy of the GNU General Public License is attached to this
  * source distribution for its full text.
  *
  * Visit http://goaccess.prosoftcorp.com for new releases.
@@ -26,6 +26,7 @@
 #define _FILE_OFFSET_BITS 64
 
 #define ARRAY_SIZE(a) (sizeof(a) / sizeof(a[0]))
+#define min(a, b) (((a) < (b)) ? (a) : (b))
 
 #define _XOPEN_SOURCE 700
 #define STDIN_FILENO  0
@@ -125,9 +126,8 @@ draw_header (WINDOW * win, char *header, int x, int y, int w, int color)
 void
 update_header (WINDOW * header_win, int current)
 {
-   int row = 0, col = 0;
+   int col = getmaxx (stdscr);
 
-   getmaxyx (stdscr, row, col);
    wattron (header_win, COLOR_PAIR (BLUE_GREEN));
    wmove (header_win, 0, 30);
    mvwprintw (header_win, 0, col - 19, "[Active Module %d]", current);
@@ -148,8 +148,7 @@ term_size (WINDOW * main_win)
 static void
 colour_noutput (WINDOW * header_win, char *str, int y)
 {
-   int row, col;
-   getmaxyx (stdscr, row, col);
+   int col = getmaxx (stdscr);
 
    char *p;
    int x = 2, path_flag = 0;
@@ -174,14 +173,135 @@ colour_noutput (WINDOW * header_win, char *str, int y)
       wattroff (header_win, COLOR_PAIR (COL_YELLOW));
 }
 
+char *
+input_string (WINDOW * win, int pos_y, int pos_x, int max_width, char *str)
+{
+   char *s = malloc (max_width + 1), *tmp;
+   if (s == NULL)
+      error_handler (__PRETTY_FUNCTION__, __FILE__, __LINE__,
+                     "Unable to allocate memory.");
+   int pos = 0, x = 0, quit = 1, c;
+   int size_x = getmaxx (win);
+   size_x -= 4;
+
+   if (str) {
+      size_t len = min (max_width, strlen (str));
+      memcpy (s, str, len);
+      s[len] = '\0';
+
+      x = pos = 0;
+      if (strlen (s) > size_x) {
+         tmp = strdup (&s[0]);
+         tmp[size_x] = '\0';
+         mvwprintw (win, pos_y, pos_x, "%s", tmp);
+         free (tmp);
+      } else
+         mvwprintw (win, pos_y, pos_x, "%s", s);
+   } else
+      s[0] = '\0';
+
+   wmove (win, pos_y, pos_x + x);
+   wrefresh (win);
+
+   curs_set (1);
+   while (quit) {
+      c = wgetch (stdscr);
+      switch (c) {
+       case 1:
+       case 262:
+          pos = x = 0;
+          break;
+       case 5:
+       case 360:
+          if (strlen (s) > size_x) {
+             x = size_x;
+             pos = strlen (s) - size_x;
+          } else {
+             pos = 0;
+             x = strlen (s);
+          }
+          break;
+       case 7:
+       case 27:
+          pos = x = 0;
+          if (!str)
+             s[0] = '\0';
+          quit = 0;
+          break;
+       case 21:
+          s[0] = '\0';
+          pos = x = 0;
+          break;
+       case KEY_BACKSPACE:
+          if (pos + x > 0) {
+             memmove (&s[(pos + x) - 1], &s[pos + x],
+                      (max_width - (pos + x)) + 1);
+             if (pos <= 0)
+                x--;
+             else
+                pos--;
+          }
+          break;
+       case KEY_LEFT:
+          if (x > 0)
+             x--;
+          else if (pos > 0)
+             pos--;
+          break;
+       case KEY_RIGHT:
+          if ((x + pos) < strlen (s)) {
+             if (x < size_x)
+                x++;
+             else
+                pos++;
+          }
+          break;
+       case 0x0a:
+       case 0x0d:
+       case KEY_ENTER:
+          quit = 0;
+          break;
+       default:
+          if (strlen (s) == max_width)
+             break;
+          if (!isprint (c))
+             break;
+
+          if (strlen (s) == pos) {
+             s[pos + x] = c;
+             s[pos + x + 1] = '\0';
+             waddch (win, c);
+          } else {
+             memmove (&s[pos + x + 1], &s[pos + x], strlen (&s[pos + x]) + 1);
+             s[pos + x] = c;
+          }
+          if ((x + pos) < max_width) {
+             if (x < size_x)
+                x++;
+             else
+                pos++;
+          }
+      }
+      int i;
+      tmp = strdup (&s[pos > 0 ? pos : 0]);
+      tmp[min (strlen (tmp), size_x)] = '\0';
+      for (i = strlen (tmp); i < size_x; i++)
+         mvwprintw (win, pos_y, pos_x + i, "%s", " ");
+      mvwprintw (win, pos_y, pos_x, "%s", tmp);
+      free (tmp);
+
+      wmove (win, pos_y, pos_x + x);
+      wrefresh (win);
+   }
+   curs_set (0);
+   return s;
+}
+
 void
 display_general (WINDOW * header_win, struct logger *logger, char *ifile)
 {
-   int row, col;
-   char *head_desc =
-      " General Summary - Overall Analysed Requests - Unique totals";
-   getmaxyx (stdscr, row, col);
-   draw_header (header_win, head_desc, 0, 0, col, 1);
+   int col = getmaxx (stdscr);
+   draw_header (header_win, T_HEAD, 0, 0, col, 1);
 
    /* general stats */
    char *bw, *size;
@@ -191,10 +311,7 @@ display_general (WINDOW * header_win, struct logger *logger, char *ifile)
    } else
       size = alloc_string ("N/A");
 
-   if (bandwidth_flag)
-      bw = filesize_str ((float) req_size);
-   else
-      bw = alloc_string ("N/A");
+   bw = filesize_str ((float) req_size);
 
    char *line1, *line2, *line3;
    const char *format_line1 = "%-15s %-9d %-15s %-9d %-10s %-9d %-3s %s";
@@ -254,7 +371,7 @@ display_general (WINDOW * header_win, struct logger *logger, char *ifile)
    free (bw);
 }
 
-static char *
+char *
 ht_bw_str (GHashTable * ht, const char *key)
 {
    gpointer value_ptr;
@@ -272,7 +389,7 @@ create_graphs (WINDOW * main_win, struct struct_display **s_display,
 {
    char buf[12] = "";           /* date */
    float l_bar, scr_cal, orig_cal;
-   int x, y, xx, r, col, row;
+   int x, y, xx, r;
    struct tm tm;
 
    GHashTable *hash_table = NULL;
@@ -280,7 +397,8 @@ create_graphs (WINDOW * main_win, struct struct_display **s_display,
    memset (&tm, 0, sizeof (tm));
 
    getyx (main_win, y, x);
-   getmaxyx (stdscr, row, col);
+   x = x;                       /* Get rid of "set but not used" lint warning */
+   int col = getmaxx (stdscr);
 
    switch (module) {
     case UNIQUE_VISITORS:
@@ -319,13 +437,11 @@ create_graphs (WINDOW * main_win, struct struct_display **s_display,
       strftime (buf, sizeof (buf), "%d/%b/%Y", &tm);
       mvwprintw (main_win, y, 18 + padding_rx, "%s", buf);
       /* bandwidth */
-      if (bandwidth_flag) {
-         char *bw = ht_bw_str (ht_date_bw, buf);
-         wattron (main_win, A_BOLD | COLOR_PAIR (COL_BLACK));
-         mvwprintw (main_win, y, 31 + padding_rx, "%9s", bw);
-         wattroff (main_win, A_BOLD | COLOR_PAIR (COL_BLACK));
-         free (bw);
-      }
+      char *bw = ht_bw_str (ht_date_bw, s_display[i]->data);
+      wattron (main_win, A_BOLD | COLOR_PAIR (COL_BLACK));
+      mvwprintw (main_win, y, 31 + padding_rx, "%9s", bw);
+      wattroff (main_win, A_BOLD | COLOR_PAIR (COL_BLACK));
+      free (bw);
    } else if (s_display[i]->module == 9)
       /* HTTP status codes */
       mvwprintw (main_win, y, 18 + padding_rx, "%s\t%s",
@@ -345,7 +461,7 @@ create_graphs (WINDOW * main_win, struct struct_display **s_display,
    if (s_display[i]->module == 9)
       return;
 
-   if (bandwidth_flag && s_display[i]->module == 1)
+   if (s_display[i]->module == 1)
       padding_gx = 44;
    else
       padding_gx = 36;
@@ -378,8 +494,8 @@ data_by_total_hits (WINDOW * main_win, int pos_y, struct logger *logger,
                     struct struct_display **s_display, int i)
 {
    float h = 0, t = 0;
-   int padding_rx = 0, magn = 0, col, row;
-   getmaxyx (main_win, row, col);
+   int padding_rx = 0, magn = 0;
+   int col = getmaxx (main_win);
 
    if (s_display[i]->hits == 0)
       return;
@@ -396,7 +512,7 @@ data_by_total_hits (WINDOW * main_win, int pos_y, struct logger *logger,
    char *c_data = s_display[i]->data;   /* current m_data */
    int c_mod = s_display[i]->module;    /* current module */
 
-   if (bandwidth_flag && (c_mod == REQUESTS || c_mod == REQUESTS_STATIC)) {
+   if ((c_mod == REQUESTS) || (c_mod == REQUESTS_STATIC)) {
       char *bw = ht_bw_str (ht_file_bw, c_data);
       wattron (main_win, A_BOLD | COLOR_PAIR (COL_BLACK));
       mvwprintw (main_win, pos_y, 18 + padding_rx, "%9s", bw);
@@ -430,15 +546,15 @@ data_by_total_hits (WINDOW * main_win, int pos_y, struct logger *logger,
    wattroff (main_win, A_BOLD | COLOR_PAIR (COL_BLACK));
 }
 
-/* ###NOTE: Modules 6, 7 are based on module 1 totals 
+/* ###NOTE: Modules 6, 7 are based on module 1 totals
    this way we avoid the overhead of adding them up */
 void
 display_content (WINDOW * main_win, struct struct_display **s_display,
                  struct logger *logger, struct scrolling scrolling)
 {
-   int i, x, y, max = 0, until = 0, start = 0, pos_y = 0;
+   int i, max = 0, until = 0, start = 0, pos_y = 0;
 
-   getmaxyx (main_win, y, x);
+   int x = getmaxx (main_win);
    getmaxyx (stdscr, term_h, term_w);
 
    if (term_h < MIN_HEIGHT || term_w < MIN_WIDTH)
@@ -499,15 +615,16 @@ display_content (WINDOW * main_win, struct struct_display **s_display,
    }
 }
 
-/* ###NOTE: Modules 6, 7 are based on module 1 totals 
+/* ###NOTE: Modules 6, 7 are based on module 1 totals
    this way we avoid the overhead of adding them up */
 void
 do_scrolling (WINDOW * main_win, struct struct_display **s_display,
               struct logger *logger, struct scrolling *scrolling, int cmd)
 {
-   int cur_y, cur_x, y, x, max = 0;
-   getmaxyx (main_win, y, x);
+   int cur_y, cur_x, max = 0;
+   int x = getmaxx (main_win);
    getyx (main_win, cur_y, cur_x);      /* cursor */
+   cur_x = cur_x;               /* Get rid of "set but not used" lint warning */
 
    int i = real_size_y + scrolling->init_scrl_main_win;
    int j = scrolling->init_scrl_main_win - 1;
@@ -619,8 +736,7 @@ static void
 load_help_popup_content (WINDOW * inner_win, int where,
                          struct scrolling *scrolling)
 {
-   int y, x;
-   getmaxyx (inner_win, y, x);
+   int y = getmaxy (inner_win);
 
    switch (where) {
        /* scroll down */
@@ -701,8 +817,7 @@ static void
 scrl_agent_win (WINDOW * inner_win, int where, struct scrolling *scrolling,
                 struct struct_agents *s_agents, size_t alloc)
 {
-   int y, x;
-   getmaxyx (inner_win, y, x);
+   int y = getmaxy (inner_win);
 
    switch (where) {
        /* scroll down */
@@ -737,7 +852,7 @@ scrl_agent_win (WINDOW * inner_win, int where, struct scrolling *scrolling,
 }
 
 /* split agent str if length > max or if '|' is found */
-static void
+void
 split_agent_str (char *ptr_value, struct struct_agents *s_agents, size_t max)
 {
    char *holder;
@@ -971,7 +1086,7 @@ load_schemes_win (WINDOW * schemes_win, size_t startx)
    return;
 }
 
-static char *
+char *
 get_browser_type (char *line)
 {
    char *p;
@@ -1055,7 +1170,7 @@ static char *
 set_month_data_unique_visitors (char *data)
 {
    char buf[DATELEN] = "";
-   char *buffer_month;
+   char *buffer_month, *date = NULL, *end = NULL;
    struct tm tm;
 
    memset (&tm, 0, sizeof (tm));
@@ -1063,51 +1178,38 @@ set_month_data_unique_visitors (char *data)
    if ((data == NULL))
       return alloc_string ("-");
 
+   date = clean_date (data);
+   end = strptime (date, "%Y%m", &tm);
+   if (end == NULL || *end != '\0')
+      return alloc_string ("-");
+
+   if (strftime (buf, sizeof (buf), "%b/%Y", &tm) <= 0)
+      return alloc_string ("-");
+
    buffer_month = (char *) malloc (sizeof (char) * (DATELEN));
    if (buffer_month == NULL)
       error_handler (__PRETTY_FUNCTION__, __FILE__, __LINE__,
                      "Unable to allocate memory");
-
-   strptime (data, "%Y%m", &tm);
-   if (strftime (buf, sizeof (buf), "%b/%Y", &tm) <= 0)
-      *buf = 0;
-   strftime (buf, sizeof (buf), "%b/%Y", &tm);
    sprintf (buffer_month, "%s", buf);
-
+   if (date)
+      free (date);
    return buffer_month;
 }
 
 static char *
-set_nobw_data_unique_visitors (char *buf, char *graph)
-{
-   char *data;
-   int len;
-
-   if ((buf == NULL) || (graph == NULL))
-      return alloc_string ("-");
-
-   len = snprintf (NULL, 0, "|`- %s %s", buf, graph);
-   data = malloc (len + 2);
-   if (data == NULL)
-      error_handler (__PRETTY_FUNCTION__, __FILE__, __LINE__,
-                     "Unable to allocate memory");
-   sprintf (data, "|`- %s %s", buf, graph);
-
-   return data;
-}
-
-static char *
-set_bw_data_unique_visitors (char *buf, char *graph)
+set_bw_data_unique_visitors (char *date, char *graph)
 {
+   char buf[DATELEN] = "";
    char *bw, *w_bw;
    gpointer value_ptr;
    int len;
    long long *ptr_value;
 
-   value_ptr = g_hash_table_lookup (ht_date_bw, buf);
+   value_ptr = g_hash_table_lookup (ht_date_bw, date);
    if (value_ptr == NULL)
       return alloc_string ("-");
 
+   convert_date (buf, date, DATELEN);
    ptr_value = (long long *) value_ptr;
    bw = filesize_str (*ptr_value);
    len = snprintf (NULL, 0, "|`- %s %9s %s", buf, bw, graph);
@@ -1125,21 +1227,16 @@ static ITEM **
 get_menu_items (struct struct_holder **s_holder, struct logger *logger,
                 int choices, int sort, WINDOW * my_menu_win, int max)
 {
-   char buf[DATELEN] = "";
    char *b_version = NULL, *o_version = NULL;
    char *hits = NULL, *graph = NULL;
    ITEM **items;
-   struct tm tm;
 
    char *bw, *w_bw, *status_code, *token, *status_str;
    gpointer value_ptr;
    int i;
    long long *ptr_value;
-   size_t y, x;
 
-   memset (&tm, 0, sizeof (tm));
-
-   getmaxyx (my_menu_win, y, x);
+   size_t x = getmaxx (my_menu_win);
    /* sort struct prior to display */
    if (logger->current_module == BROWSERS || logger->current_module == OS)
       qsort (s_holder, choices, sizeof *s_holder, struct_cmp_asc);
@@ -1156,17 +1253,9 @@ get_menu_items (struct struct_holder **s_holder, struct logger *logger,
       switch (logger->current_module) {
        case UNIQUE_VISITORS:
           hits = convert_hits_to_string (s_holder[i]->hits);
-          if (strchr (s_holder[i]->data, '|') == NULL && bandwidth_flag) {
+          if (strchr (s_holder[i]->data, '|') == NULL) {
              graph = subgraphs (s_holder, s_holder[i]->hits, x, max);
-             convert_date (buf, s_holder[i]->data, DATELEN);
-             data = set_bw_data_unique_visitors (buf, graph);
-             items[i] = new_item (hits, data);
-             free (graph);
-             break;
-          } else if (strchr (s_holder[i]->data, '|') == NULL) {
-             graph = subgraphs (s_holder, s_holder[i]->hits, x + 8, max);
-             convert_date (buf, s_holder[i]->data, DATELEN);
-             data = set_nobw_data_unique_visitors (buf, graph);
+             data = set_bw_data_unique_visitors (s_holder[i]->data, graph);
              items[i] = new_item (hits, data);
              free (graph);
              break;
@@ -1177,11 +1266,6 @@ get_menu_items (struct struct_holder **s_holder, struct logger *logger,
        case REQUESTS:
        case REQUESTS_STATIC:
        case HOSTS:
-          if (!bandwidth_flag) {
-             hits = convert_hits_to_string (s_holder[i]->hits);
-             items[i] = new_item (hits, s_holder[i]->data);
-             break;
-          }
           if (logger->current_module == HOSTS)
              value_ptr = g_hash_table_lookup (ht_host_bw, s_holder[i]->data);
           else
@@ -1439,7 +1523,7 @@ load_popup (WINDOW * my_menu_win, struct struct_holder **s_holder,
    /* set it 0 for the next g_hash_table_foreach() */
    iter_ctr = 0;
 
-   /* again, letting the user to set the max number 
+   /* again, letting the user to set the max number
     * might be a better way to go */
    if (logger->current_module == BROWSERS)
       choices = g_hash_table_size (hash_table);
